from enum import Enum, auto
from . import Command
from librusapi import Grades
from ..lang import write, flavor_text
from datetime import datetime, timedelta
from prettytable import PrettyTable


class Grade(Command):
    """
    All things concering the 'grade' command.
    """

    class Filter(Enum):
        """
        Different options for 'grade' command.
        """

        recent = auto()

    def __init__(self):
        super().__init__()
        self.grades = None
        self.weeks = 1
        self.filter = None
        self.filters = {self.Filter.recent: self.grades_recent}
        fields = (
            flavor_text("Name", fore="CYAN", style="BRIGHT"),
            flavor_text("Date", fore="YELLOW", style="BRIGHT"),
            flavor_text("Category", fore="MAGENTA", style="BRIGHT"),
            flavor_text("Value", fore="GREEN", style="BRIGHT"),
        )
        self.table = PrettyTable(fields)

    def parse(self, argv):
        self.grades = Grades(self.token)
        if len(argv) >= 1:
            try:
                self.filter = self.Filter[argv[0]]
            except KeyError:
                try:
                    self.weeks = int(argv[0])
                except ValueError:
                    write(f"'{argv[0]}' is not a valid filter!", fore="RED")

            try:
                self.weeks = int(argv[1])
            except (ValueError, IndexError):
                pass

        if self.filter:
            self.filters[self.filter]()
        else:
            # * Default
            self.filters[self.Filter.recent]()

    def colorize_grade(self, grade: str) -> str:
        colors = ("RED", "YELLOW", "BLUE", "CYAN", "GREEN", "MAGENTA")
        try:
            val = int(grade[0]) - 1
            return flavor_text(
                grade, colors[val], style="BRIGHT" if val == 5 else "NORMAL"
            )
        except (ValueError, IndexError):
            return flavor_text(grade, "WHITE", style="BRIGHT")

    def grades_recent(self):
        """
        Prints a table with last 'self.weeks' weeks of grades.
        Results are grouped by lesson.
        """
        lessons = self.grades.get()
        for lesson in lessons:
            for grade in lesson.grades:
                if datetime.now() - grade.date <= timedelta(days=7 * self.weeks):
                    row = (
                        flavor_text(lesson.name, "CYAN"),
                        flavor_text(grade.date.strftime("%a, %d-%b-%Y"), "YELLOW"),
                        flavor_text(grade.category, "MAGENTA"),
                        self.colorize_grade(grade.value),
                    )
                    self.table.add_row(row)
        print(self.table)
